static const char* op_c_source =
"/* This file is an image processing operation for GEGL                        \n"
" *                                                                            \n"
" * GEGL is free software; you can redistribute it and/or                      \n"
" * modify it under the terms of the GNU Lesser General Public                 \n"
" * License as published by the Free Software Foundation; either               \n"
" * version 3 of the License, or (at your option) any later version.           \n"
" *                                                                            \n"
" * GEGL is distributed in the hope that it will be useful,                    \n"
" * but WITHOUT ANY WARRANTY; without even the implied warranty of             \n"
" * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          \n"
" * Lesser General Public License for more details.                            \n"
" *                                                                            \n"
" * You should have received a copy of the GNU Lesser General Public           \n"
" * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.       \n"
" *                                                                            \n"
" * Copyright 2009 Øyvind Kolås <pippin@gimp.org>                            \n"
" */                                                                           \n"
"                                                                              \n"
"#include \"config.h\"                                                         \n"
"#include <glib/gi18n-lib.h>                                                   \n"
"                                                                              \n"
"#ifdef GEGL_PROPERTIES                                                        \n"
"                                                                              \n"
"property_int  (levels, _(\"Levels\"), 8)                                      \n"
"  description (_(\"number of levels per component\"))                         \n"
"  value_range (1, 64)                                                         \n"
"  ui_gamma    (2.0)                                                           \n"
"                                                                              \n"
"#else                                                                         \n"
"                                                                              \n"
"#define GEGL_OP_POINT_FILTER                                                  \n"
"#define GEGL_OP_C_SOURCE posterize.c                                          \n"
"#define GEGLV4                                                                \n"
"                                                                              \n"
"#include \"gegl-op.h\"                                                        \n"
"                                                                              \n"
"#ifndef RINT                                                                  \n"
"#define RINT(a) ((gint)(a+0.5))                                               \n"
"#endif                                                                        \n"
"                                                                              \n"
"                                                                              \n"
"static gboolean process (GeglOperation       *operation,                      \n"
"                         void                *in_buf,                         \n"
"                         void                *out_buf,                        \n"
"                         glong                samples,                        \n"
"                         const GeglRectangle *roi,                            \n"
"                         gint                 level)                          \n"
"{                                                                             \n"
"  GeglProperties *o      = GEGL_PROPERTIES (operation);                       \n"
"  gfloat     *src    = in_buf;                                                \n"
"  gfloat     *dest   = out_buf;                                               \n"
"  gfloat      levels = o->levels;                                             \n"
"                                                                              \n"
"  while (samples--)                                                           \n"
"    {                                                                         \n"
"      gint i;                                                                 \n"
"      for (i=0; i < 3; i++)                                                   \n"
"        dest[i] = RINT (src[i]   * levels) / levels;                          \n"
"      dest[3] = src[3];                                                       \n"
"                                                                              \n"
"      src  += 4;                                                              \n"
"      dest += 4;                                                              \n"
"    }                                                                         \n"
"                                                                              \n"
"  return TRUE;                                                                \n"
"}                                                                             \n"
"                                                                              \n"
"#include \"opencl/gegl-cl.h\"                                                 \n"
"#include \"opencl/posterize.cl.h\"                                            \n"
"                                                                              \n"
"static GeglClRunData *cl_data = NULL;                                         \n"
"                                                                              \n"
"static gboolean                                                               \n"
"cl_process (GeglOperation       *operation,                                   \n"
"            cl_mem              in,                                           \n"
"            cl_mem              out,                                          \n"
"            size_t              global_worksize,                              \n"
"            const GeglRectangle *roi,                                         \n"
"            gint                level)                                        \n"
"{                                                                             \n"
"  GeglProperties *o      = GEGL_PROPERTIES (operation);                       \n"
"  cl_float    levels = o->levels;                                             \n"
"                                                                              \n"
"  if (!cl_data)                                                               \n"
"    {                                                                         \n"
"      const char *kernel_name[] = {\"cl_posterize\",                          \n"
"                                   NULL};                                     \n"
"      cl_data = gegl_cl_compile_and_build (posterize_cl_source, kernel_name); \n"
"    }                                                                         \n"
"                                                                              \n"
"  if (!cl_data)                                                               \n"
"    return 1;                                                                 \n"
"                                                                              \n"
"  {                                                                           \n"
"  cl_int cl_err = 0;                                                          \n"
"                                                                              \n"
"  gegl_cl_set_kernel_args (cl_data->kernel[0],                                \n"
"                           sizeof(cl_mem),   &in,                             \n"
"                           sizeof(cl_mem),   &out,                            \n"
"                           sizeof(cl_float), &levels,                         \n"
"                           NULL);                                             \n"
"  CL_CHECK;                                                                   \n"
"                                                                              \n"
"  cl_err = gegl_clEnqueueNDRangeKernel (gegl_cl_get_command_queue (),         \n"
"                                        cl_data->kernel[0], 1,                \n"
"                                        NULL, &global_worksize, NULL,         \n"
"                                        0, NULL, NULL);                       \n"
"  CL_CHECK;                                                                   \n"
"  }                                                                           \n"
"                                                                              \n"
"  return  FALSE;                                                              \n"
"                                                                              \n"
"error:                                                                        \n"
"  return TRUE;                                                                \n"
"}                                                                             \n"
"                                                                              \n"
"static void                                                                   \n"
"gegl_op_class_init (GeglOpClass *klass)                                       \n"
"{                                                                             \n"
"  GeglOperationClass            *operation_class;                             \n"
"  GeglOperationPointFilterClass *point_filter_class;                          \n"
"                                                                              \n"
"  operation_class    = GEGL_OPERATION_CLASS (klass);                          \n"
"  point_filter_class = GEGL_OPERATION_POINT_FILTER_CLASS (klass);             \n"
"                                                                              \n"
"  operation_class->opencl_support = TRUE;                                     \n"
"  point_filter_class->process     = process;                                  \n"
"  point_filter_class->cl_process  = cl_process;                               \n"
"                                                                              \n"
"  gegl_operation_class_set_keys (operation_class,                             \n"
"    \"name\",        \"gegl:posterize\",                                      \n"
"    \"title\",       _(\"Posterize\"),                                        \n"
"    \"categories\" , \"color\",                                               \n"
"    \"description\",                                                          \n"
"       _(\"Reduces the number of levels in each color component of the image.\"),\n"
"       NULL);                                                                 \n"
"                                                                              \n"
"}                                                                             \n"
"                                                                              \n"
"#endif                                                                        \n"
;
