static const char* op_c_source =
"/* This file is an image processing operation for GEGL                        \n"
" *                                                                            \n"
" * GEGL is free software; you can redistribute it and/or                      \n"
" * modify it under the terms of the GNU Lesser General Public                 \n"
" * License as published by the Free Software Foundation; either               \n"
" * version 3 of the License, or (at your option) any later version.           \n"
" *                                                                            \n"
" * GEGL is distributed in the hope that it will be useful,                    \n"
" * but WITHOUT ANY WARRANTY; without even the implied warranty of             \n"
" * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          \n"
" * Lesser General Public License for more details.                            \n"
" *                                                                            \n"
" * You should have received a copy of the GNU Lesser General Public           \n"
" * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.       \n"
" *                                                                            \n"
" * Copyright 2006 Øyvind Kolås <pippin@gimp.org>                            \n"
" * Copyright 2006 Geert Jordaens <geert.jordaens@telenet.be>                  \n"
" */                                                                           \n"
"                                                                              \n"
"#include \"config.h\"                                                         \n"
"#include <glib/gi18n-lib.h>                                                   \n"
"                                                                              \n"
"#ifdef GEGL_PROPERTIES                                                        \n"
"                                                                              \n"
"property_string (values, _(\"Values\"), \"\")                                 \n"
"    description(_(\"list of <number>s\"))                                     \n"
"                                                                              \n"
"#else                                                                         \n"
"                                                                              \n"
"#define GEGL_OP_POINT_FILTER                                                  \n"
"#define GEGL_OP_C_SOURCE svg-saturate.c                                       \n"
"                                                                              \n"
"#include \"gegl-op.h\"                                                        \n"
"#include <math.h>                                                             \n"
"#include <stdlib.h>                                                           \n"
"                                                                              \n"
"                                                                              \n"
"static void prepare (GeglOperation *operation)                                \n"
"{                                                                             \n"
"  const Babl *format = babl_format (\"RaGaBaA float\");                       \n"
"                                                                              \n"
"  gegl_operation_set_format (operation, \"input\", format);                   \n"
"  gegl_operation_set_format (operation, \"output\", format);                  \n"
"}                                                                             \n"
"                                                                              \n"
"static gboolean                                                               \n"
"process (GeglOperation       *op,                                             \n"
"         void                *in_buf,                                         \n"
"         void                *out_buf,                                        \n"
"         glong                n_pixels,                                       \n"
"         const GeglRectangle *roi,                                            \n"
"         gint                 level)                                          \n"
"{                                                                             \n"
"  GeglProperties *o = GEGL_PROPERTIES (op);                                   \n"
"  gfloat     *in = in_buf;                                                    \n"
"  gfloat     *out = out_buf;                                                  \n"
"  gfloat     *m;                                                              \n"
"                                                                              \n"
"  gfloat ma[25] = { 1.0, 0.0, 0.0, 0.0, 0.0,                                  \n"
"                    0.0, 1.0, 0.0, 0.0, 0.0,                                  \n"
"                    0.0, 0.0, 1.0, 0.0, 0.0,                                  \n"
"                    0.0, 0.0, 0.0, 1.0, 0.0,                                  \n"
"                    0.0, 0.0, 0.0, 0.0, 1.0};                                 \n"
"  char         *endptr;                                                       \n"
"  gfloat        value;                                                        \n"
"  const gchar  *delimiter = \",\";                                            \n"
"  const gchar  *delimiters = \" \";                                           \n"
"  gchar       **values;                                                       \n"
"  glong         i;                                                            \n"
"                                                                              \n"
"  m = ma;                                                                     \n"
"                                                                              \n"
"  if ( o->values != NULL )                                                    \n"
"    {                                                                         \n"
"      g_strstrip(o->values);                                                  \n"
"      g_strdelimit (o->values, delimiters, *delimiter);                       \n"
"      values = g_strsplit (o->values, delimiter, 1);                          \n"
"      if (values[0] != NULL)                                                  \n"
"        {                                                                     \n"
"          value = g_ascii_strtod(values[0], &endptr);                         \n"
"          if (endptr != values[0])                                            \n"
"            if ( value >= 0.0 && value <= 1.0 )                               \n"
"              {                                                               \n"
"                 ma[0]  = 0.213 + 0.787 * value;                              \n"
"                 ma[1]  = 0.715 - 0.715 * value;                              \n"
"                 ma[2]  = 0.072 - 0.072 * value;                              \n"
"                 ma[5]  = 0.213 - 0.213 * value;                              \n"
"                 ma[6]  = 0.715 + 0.285 * value;                              \n"
"                 ma[7]  = 0.072 - 0.072 * value;                              \n"
"                 ma[10] = 0.213 - 0.213 * value;                              \n"
"                 ma[11] = 0.715 - 0.715 * value;                              \n"
"                 ma[12] = 0.072 + 0.928 * value;                              \n"
"              }                                                               \n"
"        }                                                                     \n"
"      g_strfreev(values);                                                     \n"
"    }                                                                         \n"
"                                                                              \n"
"  for (i=0; i<n_pixels; i++)                                                  \n"
"    {                                                                         \n"
"      out[0] =  m[0]  * in[0] +  m[1]  * in[1] + m[2]  * in[2] + m[3]  * in[3] + m[4];\n"
"      out[1] =  m[5]  * in[0] +  m[6]  * in[1] + m[7]  * in[2] + m[8]  * in[3] + m[9];\n"
"      out[2] =  m[10] * in[0] +  m[11] * in[1] + m[12] * in[2] + m[13] * in[3] + m[14];\n"
"      out[3] =  m[15] * in[0] +  m[16] * in[1] + m[17] * in[2] + m[18] * in[3] + m[19];\n"
"      in  += 4;                                                               \n"
"      out += 4;                                                               \n"
"    }                                                                         \n"
"                                                                              \n"
"  return TRUE;                                                                \n"
"}                                                                             \n"
"                                                                              \n"
"                                                                              \n"
"static void                                                                   \n"
"gegl_op_class_init (GeglOpClass *klass)                                       \n"
"{                                                                             \n"
"  GeglOperationClass            *operation_class;                             \n"
"  GeglOperationPointFilterClass *point_filter_class;                          \n"
"                                                                              \n"
"  operation_class    = GEGL_OPERATION_CLASS (klass);                          \n"
"  point_filter_class = GEGL_OPERATION_POINT_FILTER_CLASS (klass);             \n"
"                                                                              \n"
"  point_filter_class->process = process;                                      \n"
"  operation_class->prepare = prepare;                                         \n"
"                                                                              \n"
"  gegl_operation_class_set_keys (operation_class,                             \n"
"    \"name\"       , \"gegl:svg-saturate\",                                   \n"
"    \"categories\" , \"compositors:svgfilter\",                               \n"
"    \"description\", _(\"SVG color matrix operation svg_saturate\"),          \n"
"    NULL);                                                                    \n"
"}                                                                             \n"
"                                                                              \n"
"#endif                                                                        \n"
;
